from __future__ import with_statement

import os
import sys

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension
from distutils.command.build_ext import build_ext
from distutils.errors import CCompilerError, DistutilsExecError, \
    DistutilsPlatformError


# This script is borrowed and heavily adapted from the simplejson setup.py
# https://github.com/simplejson/simplejson/blob/master/setup.py

IS_PYPY = hasattr(sys, 'pypy_translation_info')

ext_errors = (CCompilerError, DistutilsExecError, DistutilsPlatformError)


class BuildFailed(Exception):
    pass


class ve_build_ext(build_ext):
    # This class allows C extension building to fail.

    def run(self):
        try:
            build_ext.run(self)
        except DistutilsPlatformError:
            raise BuildFailed()

    def build_extension(self, ext):
        try:
            build_ext.build_extension(self, ext)
        except ext_errors:
            raise BuildFailed()


src_dir = "src/"
c_extension = Extension(
    "entropy_c",
    sources=[
        src_dir + "entropy.c",
        src_dir + "entropy_c_module.c",
    ],
    include_dirs=[src_dir],
)


def run_setup(setup_kwargs):
    new_kwargs = dict(
        setup_kwargs, 
        ext_modules=[c_extension],
        cmdclass=dict(build_ext=ve_build_ext)
    )

    setup(**new_kwargs)


DISABLE_SPEEDUPS = IS_PYPY or os.environ.get('DISABLE_SPEEDUPS') == '1'
CIBUILDWHEEL = os.environ.get('CIBUILDWHEEL') == '1'
REQUIRE_SPEEDUPS = CIBUILDWHEEL or os.environ.get('REQUIRE_SPEEDUPS') == '1'


def attempt_c_build(setup_kwargs):
    try:
        # This might be problematic. When we do this build, we end up with 
        # library files at eg build\lib.win-amd64-cpython-313\seamless_entropy
        # When we fallback to doing the Python only build, poetry will retrieve 
        # these built files, instead of the newly built output of the Python only
        # build at eg build\lib
        # The former *should* be the same as the latter but since the former are
        # the result of an abortive build, who knows what they will be?
        run_setup(setup_kwargs)
        return True
    except BuildFailed:
        return False


def build(setup_kwargs):
    print("Calling the build script")

    if DISABLE_SPEEDUPS:
        print("Building without C extension compilation")
        setup_kwargs.update(
            {
                'has_ext_modules': lambda : False
            }
        )
    else:
        c_buildable = attempt_c_build(setup_kwargs)
        if c_buildable:
            print("Building with C extensions compiled.")
            setup_kwargs.update(
                {
                    "ext_modules": [c_extension],
                }
            )
        elif REQUIRE_SPEEDUPS:
                raise
        else:
            print("WARNING: The C extension could not be compiled, speedups are not enabled.")
            setup_kwargs.update(
                {
                    # This override means that setuptools does the right thing
                    # (builds with a name tagged 'any' platform). However poetry
                    # does the wrong thing and gives the packaged wheel a platform-
                    # specific name. This is a poetry limitation:
                    # https://github.com/python-poetry/poetry/issues/3594
                    # We keep the correct setuptools behavior here, so that it's 
                    # easier to fix the whole thing if there is ever a poetry
                    # fix or workaround.
                    'has_ext_modules': lambda : False,
                }
            )




def no_op_decorator(f):
    f._platform = "Pure Python"
    return f


def numba_wrapper(f):
    f._platform = "Numba"
    f = numba.jit(f)
    return f


try:
    import numba
    optionally_numba = numba.jit
except ImportError:
    optionally_numba = no_op_decorator


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['seamless_entropy']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'seamless-entropy',
    'version': '0.2.4',
    'description': 'An experimental package intended to test adaptive dependencies for numeric code.',
    'long_description': "# Seamless entropy\n\nThe purpose of this package is to test and demonstrate how to create Python modules which can correctly fallback to supported functionality if dependencies such as C compilation, scipy, numba, etc., are not available.\n\nThis packages exports a single function, `binary_entropy`, which returns the entropy of a probability, expressed in bits (rather than using the natural logarithm, as is customary in most mathematical applications).\n\n```\n>>> from seamless_entropy import binary_entropy\n>>> binary_entropy(0.25)\n0.5\n\n```\n\n## Choosing an implementation\nThe package will choose from one of several implementations of this function, depending on the environment \n\n a. at build time, and \n b. at run time.\n\nFor example, if C compilation is supported and enabled, either when building a wheel, or when installing from source, an implementation in a C extension module will be built and used in preference.\n\nIf numba is present, either because the package was installed as `seamless_entropy[numba]`, or because it happened to be installed anyway, the Python function will be jitted before use.\n\nIf scipy is present, again either by installing `seamless_entropy[scipy]` or incidentally, an implementation which calls a scipy function will be preferred.\n\n## Controlling the build\nWhen building (this can mean either building a redistributable wheel, or installing from source), you can set environment variables to control how the package is built.\n\n- DISABLE_SPEEDUPS: setting this to '1' means that C code will not be built even if that is supported.\n- REQUIRE_SPEEDUPS: setting this to '1' means that if C code cannot be built, the \n\n## TODO\nIt would be nice to have a Cython implementation. Note that Cython support depends on C support; if Cython is available than native C should work, but not the converse. Thus the Cython implementation should take priority over the C one.\n\nHow about configurable order of preference for different platforms?\n\n## Installation errors\nThe aim with this package is to solve in one place all the packaging issues which arise from a package which optionally includes faster and more efficient versions. This technology is intended to be re-used in other packages. It would be nice to make this package as robust as possible on various Python platforms. If you encounter any problems installing or running this package, on any Python setup no matter how weird, please let me know! I would love to know about your environment so that I can make this package work on it.",
    'author': 'Jack Grahl',
    'author_email': 'jack.grahl@gmail.com',
    'maintainer': 'Jack Grahl',
    'maintainer_email': 'jack.grahl@gmail.com',
    'url': 'https://github.com/jwg4/seamless_entropy',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)



try:
    from entropy_c import binary_entropy
    
except ImportError:
    try:
        import scipy # type: ignore
        from .use_scipy import binary_entropy
    except ImportError:
        from .pure_python import binary_entropy
